#include <stdio.h>

int main() {
	unsigned char i = 0x01, j;

	printf("%02x\n", i);
	// The left-shift operator causes the bits in shift-expression to be shifted to the left 
	// by the number of positions specified by additive-expression. The bit positions that 
	// have been vacated by the shift operation are zero-filled. 
	i = i << 2;
	printf("%02x\n", i);

	j = 0xff;
	printf("%02x\n", j);
	// The right-shift operator causes the bits in shift-expression to be shifted to the right 
	// by the number of positions specified by additive-expression. For unsigned numbers, the 
	// bit positions that have been vacated by the shift operation are zero-filled. For signed 
	// numbers, the sign bit is used to fill the vacated bit positions. 
	// In other words, if the number is positive, 0 is used, and if the number is negative, 1 is used.
	j = j >> 4;
	printf("%02x\n", j);

	// get the right-most bit
	j = 0x07;
	j = j<<7; // the 7 bits being shifted off the left end are discarded
	j = j>>7; // 7 positions are filled with zeros
	// this is not valid: j = (j<<7)>>7; For more info see: https://stackoverflow.com/questions/61958789/why-does-it-make-a-difference-if-left-and-right-shift-are-used-together-in-one-e#answers
	printf("Right-most bit of 07 (00000111): %02x\n", j);
	// second way to get the right most bit
	j = 0x07;
	j = j & 0x01; // isolate the right-most bit
	printf("Right-most bit of 07 (00000111): %02x\n", j);
	
	// get the second right-most bit 
	j = 0x1e;
	j = j & 0x02; // isolate the second right-most bit
	j = j >> 1;
	printf("2nd right-most bit of 1e (00011110): %02x\n", j);
	// get the fourth right-most bit 
	j = 0x1e;
	j = j & 0x08; // isolate the fourth right-most bit
	j = j >> 3;
	printf("4th right-most bit of 1e (00011110): %02x\n", j);
	// get the fifth right-most bit 
	j = 0x1e;
	j = j & 0x10; // isolate the fifth right-most bit
	j = j >> 4;
	printf("5th right-most bit of 1e (00011110): %02x\n", j);
	// get the sixth right-most bit 
	j = 0x1e;
	j = j & 0x20; // isolate the sixth right-most bit
	j = j >> 5;
	printf("6th right-most bit of 1e (00011110): %02x\n", j);

	return 0;
}
